@@ -7,15 +7,36 @@ try:
 except ImportError:
     import json
 
-from flask import Blueprint, render_template, abort, request, url_for, redirect, flash, Response
-from flask_peewee.filters import FilterMapping, FilterForm, FilterModelConverter
-from flask_peewee.forms import BaseModelConverter, ChosenAjaxSelectWidget, LimitedModelSelectField
+from flask import Blueprint
+from flask import Response
+from flask import abort
+from flask import flash
+from flask import redirect
+from flask import render_template
+from flask import request
+from flask import url_for
+from flask_peewee.filters import FilterForm
+from flask_peewee.filters import FilterMapping
+from flask_peewee.filters import FilterModelConverter
+from flask_peewee.forms import BaseModelConverter
+from flask_peewee.forms import ChosenAjaxSelectWidget
+from flask_peewee.forms import LimitedModelSelectField
 from flask_peewee.serializer import Serializer
-from flask_peewee.utils import get_next, PaginatedQuery, path_to_models, slugify
-from peewee import BooleanField, DateTimeField, ForeignKeyField, DateField, TextField
+from flask_peewee.utils import PaginatedQuery
+from flask_peewee.utils import get_next
+from flask_peewee.utils import path_to_models
+from flask_peewee.utils import slugify
+from peewee import BooleanField
+from peewee import DateField
+from peewee import DateTimeField
+from peewee import ForeignKeyField
+from peewee import TextField
 from werkzeug import Headers
-from wtforms import fields, widgets
-from wtfpeewee.fields import ModelSelectField, ModelSelectMultipleField, ModelHiddenField
+from wtforms import fields
+from wtforms import widgets
+from wtfpeewee.fields import ModelHiddenField
+from wtfpeewee.fields import ModelSelectField
+from wtfpeewee.fields import ModelSelectMultipleField
 from wtfpeewee.orm import model_form
 
 
@@ -357,17 +378,18 @@ class ModelAdmin(object):
         )
 
     def ajax_list(self):
-        field = request.args.get('field')
+        field_name = request.args.get('field')
         prev_page = 0
         next_page = 0
 
         try:
-            models = path_to_models(self.model, field)
+            models = path_to_models(self.model, field_name)
         except AttributeError:
             data = []
         else:
+            field = self.model._meta.fields[field_name]
             rel_model = models.pop()
-            rel_field = rel_model._meta.fields[self.foreign_key_lookups[field]]
+            rel_field = rel_model._meta.fields[self.foreign_key_lookups[field_name]]
             query = rel_model.select().order_by(rel_field)
             query_string = request.args.get('query')
             if query_string:
@@ -380,10 +402,13 @@ class ModelAdmin(object):
             if current_page < pq.get_pages():
                 next_page = current_page + 1
 
-            data = [
-                {'id': obj.get_id(), 'repr': unicode(obj)} \
-                    for obj in pq.get_list()
-            ]
+            data = []
+
+            # if the field is nullable, include the "None" option at the top of the list
+            if field.null:
+                data.append({'id': '__None', 'repr': 'None'})
+
+            data.extend([{'id': obj.get_id(), 'repr': unicode(obj)} for obj in pq.get_list()])
 
         json_data = json.dumps({'prev_page': prev_page, 'next_page': next_page, 'object_list': data})
         return Response(json_data, mimetype='application/json')
@@ -441,7 +466,7 @@ class AdminTemplateHelper(object):
         if not querystring:
             return '%s=%s' % (key, val)
         else:
-            querystring = re.sub('%s(?:[^&]+)?&?' % key, '', querystring).rstrip('&')
+            querystring = re.sub('%s(?:[^&]+)?&?' % key, '', querystring.decode('utf8')).rstrip('&')
             return ('%s&%s=%s' % (querystring, key, val)).lstrip('&')
 
     def get_verbose_name(self, model, column_name):
