@@ -4,8 +4,15 @@ import re
 import sys
 from hashlib import sha1
 
-from flask import abort, request, render_template
-from peewee import Model, DoesNotExist, SelectQuery, ForeignKeyField
+from flask import abort
+from flask import render_template
+from flask import request
+from peewee import DoesNotExist
+from peewee import ForeignKeyField
+from peewee import Model
+from peewee import SelectQuery
+
+from flask_peewee._compat import text_type
 
 
 def get_object_or_404(query_or_model, *query):
@@ -127,10 +134,11 @@ def path_to_models(model, path):
 
 # borrowing these methods, slightly modified, from django.contrib.auth
 def get_hexdigest(salt, raw_password):
-    return sha1(salt + raw_password).hexdigest()
+    data = salt + raw_password
+    return sha1(data.encode('utf8')).hexdigest()
 
 def make_password(raw_password):
-    salt = get_hexdigest(str(random.random()), str(random.random()))[:5]
+    salt = get_hexdigest(text_type(random.random()), text_type(random.random()))[:5]
     hsh = get_hexdigest(salt, raw_password)
     return '%s$%s' % (salt, hsh)
 
